import numpy as np
from adios2 import FileReader

with FileReader("cfd.bp") as s:
    # inspect variables
    vars = s.available_variables()
    for name, info in vars.items():
        print("variable_name: " + name, end=" ")
        for key, value in info.items():
            print("\t" + key + ": " + value, end=" ")
        print()
    print()

    nproc = s.read("nproc")
    print(f"nproc is {nproc} of type {type(nproc)} with ndim {nproc.ndim}")

    # read variables return a numpy array with corresponding selection
    steps = int(vars["physical_time"]["AvailableStepsCount"])
    physical_time = s.read("physical_time", step_selection=[0, steps])
    print(
        f"physical_time is {physical_time} of type {type(physical_time)} with "
        f"ndim {physical_time.ndim} shape = {physical_time.shape}"
    )

    steps = int(vars["temperature"]["AvailableStepsCount"])
    temperature = s.read("temperature", step_selection=[0, steps])
    temp_unit = s.read_attribute("temperature/unit")
    print(f"temperature array size is {temperature.size} of shape {temperature.shape}")
    print(f"temperature unit is {temp_unit} of type {type(temp_unit)}")

    steps = int(vars["pressure"]["AvailableStepsCount"])
    pressure = s.read("pressure", step_selection=[0, steps])
    press_unit = s.read_attribute("pressure/unit")

    print()



#!/usr/bin/env python3

import argparse
import glob
import sqlite3
import zlib
import yaml
from dataclasses import dataclass
from datetime import datetime
from os import chdir, getcwd, remove, stat
from os.path import exists, isdir, expanduser
from re import sub
from socket import getfqdn
from time import time_ns

# from adios2.adios2_campaign_manager import *

ADIOS_ACA_VERSION = "0.1"

@dataclass
class UserOption:
    adios_campaign_store: str = None
    hostname: str = None
    verbose: int = 0


def ReadUserConfig():
    path = expanduser("~/.config/adios2/adios2.yaml")
    opts = UserOption()
    try:
        doc = {}
        with open(path) as f:
            doc = yaml.safe_load(f)
        camp = doc.get("Campaign")
        if isinstance(camp, dict):
            for key, value in camp.items():
                if key == "campaignstorepath":
                    opts.adios_campaign_store = expanduser(value)
                if key == "hostname":
                    opts.hostname = value
                if key == "verbose":
                    opts.verbose = value
    except FileNotFoundError:
        None
    return opts


def SetupArgs():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "command",
        help="Command: create/update/delete/info/list",
        choices=["create", "update", "delete", "info", "list"],
    )
    parser.add_argument(
        "campaign", help="Campaign name or path, with .aca or without", default=None, nargs="?"
    )
    parser.add_argument("--verbose", "-v", help="More verbosity", action="count", default=0)
    parser.add_argument(
        "--campaign_store", "-s", help="Path to local campaign store", default=None
    )
    parser.add_argument("--hostname", "-n", help="Host name unique for hosts in a campaign")
    parser.add_argument("-f", "--files", nargs="+", help="Add ADIOS files manually")
    args = parser.parse_args()

    # default values
    args.user_options = ReadUserConfig()

    if args.verbose == 0:
        args.verbose = args.user_options.verbose

    if args.campaign_store is None:
        args.campaign_store = args.user_options.adios_campaign_store

    if args.campaign_store is not None:
        while args.campaign_store[-1] == "/":
            args.campaign_store = args.campaign_store[:-1]

    if args.hostname is None:
        args.hostname = args.user_options.hostname

    args.CampaignFileName = args.campaign
    if args.campaign is not None:
        if not args.campaign.endswith(".aca"):
            args.CampaignFileName += ".aca"
        if (not exists(args.CampaignFileName) and
                not args.CampaignFileName.startswith("/") and
                args.campaign_store is not None):
            args.CampaignFileName = args.campaign_store + "/" + args.CampaignFileName

    if args.files is None:
        args.LocalCampaignDir = ".adios-campaign/"

    if args.verbose > 0:
        print(f"# Verbosity = {args.verbose}")
        print(f"# Command = {args.command}")
        print(f"# Campaign File Name = {args.CampaignFileName}")
        print(f"# Campaign Store = {args.campaign_store}")
    return args


def CheckCampaignStore(args):
    if args.campaign_store is not None and not isdir(args.campaign_store):
        print("ERROR: Campaign directory " + args.campaign_store + " does not exist", flush=True)
        exit(1)


def CheckLocalCampaignDir(args):
    if not isdir(args.LocalCampaignDir):
        print(
            "ERROR: Shot campaign data '" +
            args.LocalCampaignDir +
            "' does not exist. Run this command where the code was executed.",
            flush=True,
        )
        exit(1)


def IsADIOSDataset(dataset):
    if not isdir(dataset):
        return False
    if not exists(dataset + "/" + "md.idx"):
        return False
    if not exists(dataset + "/" + "data.0"):
        return False
    return True


def compressFile(f):
    compObj = zlib.compressobj()
    compressed = bytearray()
    blocksize = 1073741824  # 1GB #1024*1048576
    len_orig = 0
    len_compressed = 0
    block = f.read(blocksize)
    while block:
        len_orig += len(block)
        cBlock = compObj.compress(block)
        compressed += cBlock
        len_compressed += len(cBlock)
        block = f.read(blocksize)
    cBlock = compObj.flush()
    compressed += cBlock
    len_compressed += len(cBlock)

    return compressed, len_orig, len_compressed


def decompressBuffer(buf: bytearray):
    data = zlib.decompress(buf)
    return data


def AddFileToArchive(args: dict, filename: str, cur: sqlite3.Cursor, dsID: int):
    compressed = 1
    try:
        f = open(filename, "rb")
        compressed_data, len_orig, len_compressed = compressFile(f)

    except IOError:
        print(f"ERROR While reading file {filename}")
        return

    statres = stat(filename)
    ct = statres.st_ctime_ns

    cur.execute(
        "insert into bpfile "
        "(bpdatasetid, name, compression, lenorig, lencompressed, ctime, data) "
        "values (?, ?, ?, ?, ?, ?, ?) "
        "on conflict (bpdatasetid, name) do update "
        "set compression = ?, lenorig = ?, lencompressed = ?, ctime = ?, data = ?",
        (
            dsID,
            filename,
            compressed,
            len_orig,
            len_compressed,
            ct,
            compressed_data,
            compressed,
            len_orig,
            len_compressed,
            ct,
            compressed_data,
        ),
    )


def AddDatasetToArchive(hostID: int, dirID: int, dataset: str, cur: sqlite3.Cursor) -> int:
    statres = stat(dataset)
    ct = statres.st_ctime_ns
    select_cmd = (
        "select rowid from bpdataset "
        f"where hostid = {hostID} and dirid = {dirID} and name = '{dataset}'"
    )
    res = cur.execute(select_cmd)
    row = res.fetchone()
    if row is not None:
        rowID = row[0]
        print(
            f"Found dataset {dataset} in database on host {hostID} "
            f"in dir {dirID}, rowid = {rowID}"
        )
    else:
        print(f"Add dataset {dataset} to archive")
        curDS = cur.execute(
            "insert into bpdataset (hostid, dirid, name, ctime) values (?, ?, ?, ?)",
            (hostID, dirID, dataset, ct),
        )
        rowID = curDS.lastrowid
        # print(
        #     f"Inserted bpdataset {dataset} in database on host {hostID}"
        #     f" in dir {dirID}, rowid = {rowID}"
        # )
    return rowID


def ProcessFiles(args: dict, cur: sqlite3.Cursor, hostID: int, dirID: int):
    for entry in args.files:
        print(f"Process entry {entry}:")
        dsID = 0
        dataset = entry
        if IsADIOSDataset(dataset):
            dsID = AddDatasetToArchive(hostID, dirID, dataset, cur)
            cwd = getcwd()
            chdir(dataset)
            mdFileList = glob.glob("*md.*")
            profileList = glob.glob("profiling.json")
            files = mdFileList + profileList
            for f in files:
                AddFileToArchive(args, f, cur, dsID)
            chdir(cwd)
        else:
            print(f"WARNING: Dataset {dataset} is not an ADIOS dataset. Skip")


def GetHostName():
    host = getfqdn()
    if host.startswith("login"):
        host = sub("^login[0-9]*\\.", "", host)
    if host.startswith("batch"):
        host = sub("^batch[0-9]*\\.", "", host)
    if args.hostname is None:
        shorthost = host.split(".")[0]
    else:
        shorthost = args.user_options.hostname
    return host, shorthost


def AddHostName(longHostName, shortHostName):
    res = cur.execute('select rowid from host where hostname = "' + shortHostName + '"')
    row = res.fetchone()
    if row is not None:
        hostID = row[0]
        print(f"Found host {shortHostName} in database, rowid = {hostID}")
    else:
        curHost = cur.execute("insert into host values (?, ?)", (shortHostName, longHostName))
        hostID = curHost.lastrowid
        print(f"Inserted host {shortHostName} into database, rowid = {hostID}")
    return hostID


def MergeDBFiles(dbfiles: list):
    # read db files here
    result = list()
    for f1 in dbfiles:
        try:
            con = sqlite3.connect(f1)
        except sqlite3.Error as e:
            print(e)

        cur = con.cursor()
        try:
            cur.execute("select  * from bpfiles")
        except sqlite3.Error as e:
            print(e)
        record = cur.fetchall()
        for item in record:
            result.append(item[0])
        cur.close()
    return result


def AddDirectory(hostID: int, path: str) -> int:
    res = cur.execute(
        "select rowid from directory where hostid = " + str(hostID) + ' and name = "' + path + '"'
    )
    row = res.fetchone()
    if row is not None:
        dirID = row[0]
        print(f"Found directory {path} with hostID {hostID} in database, rowid = {dirID}")
    else:
        curDirectory = cur.execute("insert into directory values (?, ?)", (hostID, path))
        dirID = curDirectory.lastrowid
        print(f"Inserted directory {path} into database, rowid = {dirID}")
    return dirID


def Update(args: dict, cur: sqlite3.Cursor):
    longHostName, shortHostName = GetHostName()

    hostID = AddHostName(longHostName, shortHostName)

    rootdir = getcwd()
    dirID = AddDirectory(hostID, rootdir)
    con.commit()

    ProcessFiles(args, cur, hostID, dirID)

    con.commit()


def Create(args: dict, cur: sqlite3.Cursor):
    epoch = time_ns()
    cur.execute("create table info(id TEXT, name TEXT, version TEXT, ctime INT)")
    cur.execute(
        "insert into info values (?, ?, ?, ?)",
        ("ACA", "ADIOS Campaign Archive", ADIOS_ACA_VERSION, epoch),
    )
    cur.execute("create table host" + "(hostname TEXT PRIMARY KEY, longhostname TEXT)")
    cur.execute("create table directory" + "(hostid INT, name TEXT, PRIMARY KEY (hostid, name))")
    cur.execute(
        "create table bpdataset" +
        "(hostid INT, dirid INT, name TEXT, ctime INT" +
        ", PRIMARY KEY (hostid, dirid, name))"
    )
    cur.execute(
        "create table bpfile" +
        "(bpdatasetid INT, name TEXT, compression INT, lenorig INT" +
        ", lencompressed INT, ctime INT, data BLOB" +
        ", PRIMARY KEY (bpdatasetid, name))"
    )
    Update(args, cur)


def timestamp_to_datetime(timestamp: int) -> datetime:
    digits = len(str(int(timestamp)))
    t = float(timestamp)
    if digits > 18:
        t = t / 1000000000
    elif digits > 15:
        t = t / 1000000
    elif digits > 12:
        t = t / 1000
    return datetime.fromtimestamp(t)


def Info(args: dict, cur: sqlite3.Cursor):
    res = cur.execute("select id, name, version, ctime from info")
    info = res.fetchone()
    t = timestamp_to_datetime(info[3])
    print(f"{info[1]}, version {info[2]}, created on {t}")

    res = cur.execute("select rowid, hostname, longhostname from host")
    hosts = res.fetchall()
    for host in hosts:
        print(f"hostname = {host[1]}   longhostname = {host[2]}")
        res2 = cur.execute(
            'select rowid, name from directory where hostid = "' + str(host[0]) + '"'
        )
        dirs = res2.fetchall()
        for dir in dirs:
            print(f"    dir = {dir[1]}")
            res3 = cur.execute(
                'select rowid, name, ctime from bpdataset where hostid = "' +
                str(host[0]) +
                '" and dirid = "' +
                str(dir[0]) +
                '"'
            )
            bpdatasets = res3.fetchall()
            for bpdataset in bpdatasets:
                t = timestamp_to_datetime(bpdataset[2])
                print(f"        dataset = {bpdataset[1]}     created on {t}")


def List():
    path = args.campaign
    if path is None:
        if args.campaign_store is None:
            print("ERROR: Set --campaign_store for this command")
            return 1
        path = args.campaign_store
    else:
        while path[-1] == "/":
            path = path[:-1]

    # List the local campaign store
    acaList = glob.glob(path + "/**/*.aca", recursive=True)
    if len(acaList) == 0:
        print("There are no campaign archives in  " + path)
        return 2
    else:
        startCharPos = len(path) + 1
        for f in acaList:
            print(f[startCharPos:])
    return 0


def Delete():
    if exists(args.CampaignFileName):
        print(f"Delete archive {args.CampaignFileName}")
        remove(args.CampaignFileName)
        return 0
    else:
        print(f"ERROR: archive {args.CampaignFileName} does not exist")
        return 1


if __name__ == "__main__":
    args = SetupArgs()
    CheckCampaignStore(args)

    if args.command == "list":
        exit(List())

    if args.command == "delete":
        exit(Delete())

    if args.command == "create":
        print("Create archive")
        if exists(args.CampaignFileName):
            print(f"ERROR: archive {args.CampaignFileName} already exist")
            exit(1)
    elif args.command == "update" or args.command == "info":
        print(f"{args.command} archive")
        if not exists(args.CampaignFileName):
            print(f"ERROR: archive {args.CampaignFileName} does not exist")
            exit(1)

    con = sqlite3.connect(args.CampaignFileName)
    cur = con.cursor()

    if args.command == "info":
        Info(args, cur)
    else:
        if args.files is None:
            CheckLocalCampaignDir(args)
            # List the local campaign directory
            dbFileList = glob.glob(args.LocalCampaignDir + "/*.db")
            if len(dbFileList) == 0:
                print("There are no campaign data files in  " + args.LocalCampaignDir)
                exit(2)
            args.files = MergeDBFiles(dbFileList)

        if args.command == "create":
            Create(args, cur)
        elif args.command == "update":
            Update(args, cur)

    cur.close()
    con.close()



from os import fstat

import numpy as np


def ReadMetaMetadataRecord(buf, rec, pos, fileSize):
    # Read one metametadata record
    startoffset = str(pos).rjust(8)
    # 8 bytes MetaMetaIDLen + MetaMetaInfoLen Length
    mmIDlen = np.frombuffer(buf, dtype=np.uint64, count=1, offset=pos)
    pos = pos + 8
    mmInfolen = np.frombuffer(buf, dtype=np.uint64, count=1, offset=pos)
    pos = pos + 8
    # mmid = np.frombuffer(buf, dtype=np.uint8, count=mmIDlen[0], offset=pos)
    pos = pos + int(mmIDlen[0])
    # mminfo = np.frombuffer(buf, dtype=np.uint8,
    #                        count=mmInfolen[0], offset=pos)
    pos = pos + int(mmInfolen[0])

    recs = str(rec).rjust(7)
    idlen = str(mmIDlen[0]).rjust(10)
    infolen = str(mmInfolen[0]).rjust(12)

    print(
        f" | {recs} | {startoffset} | {idlen} | {infolen} |")

    return pos


def DumpMetaMetaData(fileName):
    print("========================================================")
    print("    MetaMetadata File: " + fileName)
    print("========================================================")
    with open(fileName, "rb") as f:
        fileSize = fstat(f.fileno()).st_size
        print(f" File size = {fileSize}")
        buf = f.read(fileSize)
        print(" --------------------------------------------------")
        print(" |  Record |  Offset  |  ID length |  Info length |")
        print(" --------------------------------------------------")
        pos = 0
        rec = 0
        while (pos < fileSize - 1):
            pos = ReadMetaMetadataRecord(buf, rec, pos, fileSize)
            rec = rec + 1
        print(" --------------------------------------------------")
    return True


if __name__ == "__main__":
    print("ERROR: Utility main program is bp5dbg.py")



#
# Distributed under the OSI - approved Apache License, Version 2.0. See
# accompanying file Copyright.txt for details.
#
# TestBPWriteRead2D.py
#
#
# Created on : Mar 3rd, 2019
# Author : Jeremy Logan <lot@ornl.gov>
#          Kai Germaschewski <kai.germaschewski@unh.edu>
#          William F Godoy godoywf@ornl.gov
#

from mpi4py import MPI
import numpy as np
import adios2.bindings as adios2

# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

# User data
Nx = 10
Ny = 10

count = [Nx, Ny]
start = [rank * Nx, 0]
shape = [size * Nx, Ny]

temperatures = np.zeros(count, dtype=np.int32)

for i in range(0, Nx):
    for j in range(0, Ny):
        temperatures[i, j] = (start[0] + i) * shape[1] + (j + start[1])

# ADIOS2 read
adios = adios2.ADIOS(comm)
ioWrite = adios.DeclareIO("ioWriter")

varTemperature = ioWrite.DefineVariable(
    "temperature2D", temperatures, shape, start, count, adios2.ConstantDims
)

obpStream = ioWrite.Open("HeatMap2D_py.bp", adios2.Mode.Write)
obpStream.Put(varTemperature, temperatures)
obpStream.Close()


if rank == 0:
    ioRead = adios.DeclareIO("ioReader")
    ibpStream = ioRead.Open("HeatMap2D_py.bp", adios2.Mode.ReadRandomAccess, MPI.COMM_SELF)
    var_inTemperature = ioRead.InquireVariable("temperature2D")

    info = ibpStream.BlocksInfo("temperature2D", 0)
    assert info is not None
    assert info[0]["Start"] == "0,0"
    assert info[0]["Count"] == "10,10"
    assert info[0]["WriterID"] == "0"



#!/usr/bin/env python3
import json
import argparse
from os.path import exists


def SetupArgs():
    parser = argparse.ArgumentParser()
    parser.add_argument("--indent", "-i", type=int, default=2,
                        help="indent size, default=2")
    parser.add_argument("FILE", help="Name of the JSON file")
    args = parser.parse_args()

    # print(args)
    return args


def CheckFileName(args):
    if not exists(args.FILE):
        print("ERROR: File " + args.FILE + " does not exist", flush=True)
        exit(1)


if __name__ == "__main__":

    args = SetupArgs()
    CheckFileName(args)

    with open(args.FILE) as jsonfile:
        parsed = json.load(jsonfile)

    print(json.dumps(parsed, indent=args.indent, sort_keys=False))



import numpy as np
from adios2 import Stream

with Stream("cfd.bp", "r") as s:
    # steps comes from the stream
    for _ in s.steps():
        # track current step
        print(f"Current step is {s.current_step()}")

        # inspect variables in current step
        for name, info in s.available_variables().items():
            print("variable_name: " + name, end=" ")
            for key, value in info.items():
                print("\t" + key + ": " + value, end=" ")
            print()

        if s.current_step() == 0:
            nproc = s.read("nproc")
            print(f"nproc is {nproc} of type {type(nproc)} with ndim {nproc.ndim}")

        # read variables return a numpy array with corresponding selection
        physical_time = s.read("physical_time")
        print(f"physical_time is {physical_time} of type {type(physical_time)}")
        temperature = s.read("temperature")
        temp_unit = s.read_attribute("temperature/unit")
        print(f"temperature array size is {temperature.size} of shape {temperature.shape}")
        print(f"temperature unit is {temp_unit} of type {type(temp_unit)}")
        pressure = s.read("pressure")
        press_unit = s.read_attribute("unit", "pressure")
        print(f"pressure unit is {press_unit} of type {type(press_unit)}")
        print()



from adios2.adios import Adios

import adios2.bindings as bindings

import unittest
import numpy as np


class TestVariable(unittest.TestCase):
    def test_create_write(self):
        adios = Adios()
        with adios.declare_io("BPWriter") as io:
            temps = io.define_variable("temps", np.empty([4], dtype=np.int64))
            with io.open("pythontestvariable.bp", bindings.Mode.Write) as engine:
                temps_measures = np.array([35, 40, 30, 45], dtype=np.int64)
                engine.put(temps, temps_measures)
                self.assertEqual(temps.name(), "temps")
                self.assertEqual(temps.block_id(), 0)
                self.assertEqual(temps.count(), [])
                self.assertEqual(temps.shape(), [])
                self.assertEqual(temps.sizeof(), 8)

    def test_create_reader(self):
        adios = Adios()
        with adios.declare_io("BPWriter") as io:
            temps = io.define_variable("temps", np.empty([4], dtype=np.int64))
            with io.open("pythontestvariable.bp", bindings.Mode.Write) as engine:
                temps_measures = np.array([35, 40, 30, 45], dtype=np.int64)
                engine.put(temps, temps_measures)

        with adios.declare_io("BPReader") as reader:
            with reader.open("pythontestvariable.bp", bindings.Mode.Read) as engine:
                engine.begin_step()
                temps = reader.inquire_variable("temps")
                engine.end_step()

                self.assertEqual(temps.name(), "temps")
                self.assertEqual(temps.block_id(), 0)
                self.assertEqual(temps.count(), [])
                self.assertEqual(temps.sizeof(), 8)
                self.assertEqual(temps.steps(), 1)
                self.assertEqual(temps.steps_start(), 0)

    def test_operators(self):
        adios = Adios()
        op1 = adios.define_operator("noop", "null")
        with adios.declare_io("BPWriter") as io:
            temps = io.define_variable("temps", np.empty([4], dtype=np.int64))
            temps.add_operation(op1)
            with io.open("pythontestvariable.bp", bindings.Mode.Write) as engine:
                temps_measures = np.array([35, 40, 30, 45], dtype=np.int64)
                engine.put(temps, temps_measures)

        op2 = adios.define_operator("noop2", "null")
        with adios.declare_io("BPReader") as reader:
            with reader.open("pythontestvariable.bp", bindings.Mode.Read) as engine:
                engine.begin_step()
                temps = reader.inquire_variable("temps")
                temps.add_operation(op2)
                engine.end_step()


if __name__ == "__main__":
    unittest.main()



#
# Distributed under the OSI-approved Apache License, Version 2.0.  See
# accompanying file Copyright.txt for details.
#
# dataManWriter.py
#
#  Created on: Sept 5, 2019
#      Author: Jason Wang <wangr1@ornl.gov>
#

from mpi4py import MPI
import numpy as np
from time import sleep
from adios2 import Adios, Stream

comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

Nx = 10
Ny = 10
steps = 10000

count = [Nx, Ny]
start = [rank * Nx, 0]
shape = [size * Nx, Ny]

floatArray = np.zeros(count, dtype=np.float32)

for i in range(0, Nx):
    for j in range(0, Ny):
        floatArray[i, j] = (start[0] + i) * shape[1] + (j + start[1])

adios = Adios(comm)
io = adios.declare_io("whatever")
io.set_engine("DataMan")
io.set_parameters({"IPAddress": "127.0.0.1", "Port": "12306", "Timeout": "5"})

var = io.define_variable("FloatArray", floatArray, shape, start, count, True)

with Stream(io, "HelloDataMan", "w") as stream:
    for _ in stream.steps(steps):
        # imitating costly computation
        floatArray = floatArray + 1
        sleep(1.0)

        print("Step", stream.current_step(), floatArray)
        stream.write(var, floatArray)
        # Warning: the data of the current step is not published until
        # the next loop entry or the exit of the loop



from adios2 import Stream, LocalValueDim
from random import randint

import unittest


class TestStream(unittest.TestCase):
    def test_basic(self):
        with Stream("pythonstreamtest.bp", "w") as s:
            for _ in s.steps(10):
                # Single value string
                s.write("Outlook", "Good")
                # Global array
                s.write(
                    "temp",
                    content=[randint(15, 35), randint(15, 35), randint(15, 35)],
                    shape=[3, 1],
                    start=[0, 0],
                    count=[3, 1],
                )
                # Local Value
                s.write("Wind", [5], shape=[LocalValueDim])
                # Local Array
                s.write("Coords", [38, -46], [], [], [2])

        with Stream("pythonstreamtest.bp", "r") as s:
            for _ in s.steps():
                for var_name in s.available_variables():
                    print(f"var:{var_name}\t{s.read(var_name)}")
                    self.assertEqual(s.read("Wind", block_id=0), 5)
                    self.assertEqual(s.read("Coords", block_id=0)[0], 38)
                    self.assertEqual(s.read("Coords", block_id=0)[1], -46)

    def test_start_count(self):
        with Stream("pythonstreamtest.bp", "w") as s:
            for _ in s.steps(10):
                # Global array
                s.write(
                    "temp",
                    content=[randint(15, 35), randint(15, 35), randint(15, 35)],
                    shape=[3],
                    start=[0],
                    count=[3],
                )

        with Stream("pythonstreamtest.bp", "r") as s:
            print(s.all_blocks_info("temp"))
            self.assertEqual(len(s.all_blocks_info("temp")), 10)
            for _ in s.steps():
                for var_name in s.available_variables():
                    print(f"var:{var_name}\t{s.read(var_name)}")
                    output = s.read("temp", start=[0], count=[2])
                    self.assertEqual(len(output), 2)


if __name__ == "__main__":
    unittest.main()



#
# Distributed under the OSI-approved Apache License, Version 2.0.  See
# accompanying file Copyright.txt for details.
#
# hello-world.py : adios2 Python API example to write and read a
#                   string Variable with a greeting
#
#  Created on: 2/2/2021
#      Author: Dmitry Ganyushin ganyushindi@ornl.gov
#
import sys
from mpi4py import MPI
from adios2 import Stream, FileReader

DATA_FILENAME = "hello-world-py.bp"
# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()


def writer(greeting):
    """write a string to a bp file"""
    with Stream(DATA_FILENAME, "w", comm) as fh:
        fh.write("Greeting", greeting)
    return 0


def reader():
    """read a string from a bp file as Stream"""
    message = f"variable Greeting not found in {DATA_FILENAME}"
    with Stream(DATA_FILENAME, "r", comm) as fh:
        for _ in fh.steps():
            message = fh.read("Greeting")
    return message


def filereader():
    """read a string from a bp file using random access read mode"""
    with FileReader(DATA_FILENAME, comm) as fh:
        message = fh.read("Greeting")
    return message


def main():
    """driver function"""
    greeting = "Hello World from ADIOS2"
    writer(greeting)
    message = reader()
    print("As read from adios2.Stream: {}".format(message))
    message2 = filereader()
    print("As read from adios2.FileReader: {}".format(message2))
    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

#
# Distributed under the OSI-approved Apache License, Version 2.0.  See
# accompanying file Copyright.txt for details.
#
# TestBPWriteTypes.py: test Python numpy types in ADIOS2 File
#                      Write/Read High-Level API
#  Created on: March 12, 2018
#      Author: William F Godoy godoywf@ornl.gov

from adios2NPTypes import SmallTestData
import numpy as np
from adios2 import Adios, Stream

rank = 0
size = 1

# Test data
data = SmallTestData(rank)
nx = data.Nx

shape = [size * nx]
start = [rank * nx]
count = [nx]

adios = Adios()
io = adios.declare_io("writeh5")
io.set_engine("HDF5")

# Writer
with Stream(io, "types_np.h5", "w") as fw:
    for i in range(0, 5):
        data.update(rank, i, size)

        if rank == 0 and i == 0:
            fw.write("tag", "Testing ADIOS2 high-level API")
            fw.write("nx", data.Nx)
            fw.write("gvarI8", np.array(data.i8[0]))
            fw.write("gvarI16", np.array(data.i16[0]))
            fw.write("gvarI32", np.array(data.i32[0]))
            fw.write("gvarI64", np.array(data.i64[0]))
            fw.write("gvarU8", np.array(data.u8[0]))
            fw.write("gvarU16", np.array(data.u16[0]))
            fw.write("gvarU32", np.array(data.u32[0]))
            fw.write("gvarU64", np.array(data.u64[0]))
            fw.write("gvarR32", np.array(data.r32[0]))
            fw.write("gvarR64", np.array(data.r64[0]))

            # single value attributes
            fw.write_attribute("attrStr", "Testing single string attribute")
            fw.write_attribute("attrNx", data.Nx)
            fw.write_attribute("attrI8", np.array(data.i8[0]))
            fw.write_attribute("attrI16", np.array(data.i16[0]))
            fw.write_attribute("attrI32", np.array(data.i32[0]))
            fw.write_attribute("attrI64", np.array(data.i64[0]))
            fw.write_attribute("attrU8", np.array(data.u8[0]))
            fw.write_attribute("attrU16", np.array(data.u16[0]))
            fw.write_attribute("attrU32", np.array(data.u32[0]))
            fw.write_attribute("attrU64", np.array(data.u64[0]))
            fw.write_attribute("attrR32", np.array(data.r32[0]))
            fw.write_attribute("attrR64", np.array(data.r64[0]))

            fw.write_attribute("attrStrArray", ["string1", "string2", "string3"])
            fw.write_attribute("attrI8Array", data.i8)
            fw.write_attribute("attrI16Array", data.i16)
            fw.write_attribute("attrI32Array", data.i32)
            fw.write_attribute("attrI64Array", data.i64)
            fw.write_attribute("attrU8Array", data.u8)
            fw.write_attribute("attrU16Array", data.u16)
            fw.write_attribute("attrU32Array", data.u32)
            fw.write_attribute("attrU64Array", data.u64)
            fw.write_attribute("attrR32Array", data.r32)
            fw.write_attribute("attrR64Array", data.r64)

        fw.write("steps", "Step:" + str(i))
        fw.write("varI8", data.i8, shape, start, count)
        fw.write("varI16", data.i16, shape, start, count)
        fw.write("varI32", data.i32, shape, start, count)
        fw.write("varI64", data.i64, shape, start, count)
        fw.write("varU8", data.u8, shape, start, count)
        fw.write("varU16", data.u16, shape, start, count)
        fw.write("varU32", data.u32, shape, start, count)
        fw.write("varU64", data.u64, shape, start, count)
        fw.write("varR32", data.r32, shape, start, count)
        fw.write("varR64", data.r64, shape, start, count)

        if rank == 0 and i == 0:
            fw.write_attribute("varattrStrArray", ["varattr1", "varattr2", "varattr3"], "steps")
            fw.write_attribute("varattrI8Array", data.i8, "varI8")
            fw.write_attribute("varattrI16Array", data.i16, "varI16")
            fw.write_attribute("varattrI32Array", data.i32, "varI32")
            fw.write_attribute("varattrI64Array", data.i64, "varI64")
            fw.write_attribute("varattrU8Array", data.u8, "varU8")
            fw.write_attribute("varattrU16Array", data.u16, "varU16")
            fw.write_attribute("varattrU32Array", data.u32, "varU32")
            fw.write_attribute("varattrU64Array", data.u64, "varU64")
            fw.write_attribute("varattrR32Array", data.r32, "varR32")
            fw.write_attribute("varattrR64Array", data.r64, "varR64")
            fw.write_attribute("varattrR64Value", data.r64, "varR64")

        fw.end_step()

# Reader
data = SmallTestData(rank)

io = adios.declare_io("readh5")
io.set_engine("HDF5")

with Stream("types_np.h5", "r") as fr:
    for fr_step in fr:
        step = fr_step.current_step()
        data.update(rank, step, size)

        step_vars = fr_step.available_variables()

        #         for name, info in step_vars.items():
        #             print("variable_name: " + name)
        #             for key, value in info.items():
        #                 print("\t" + key + ": " + value)
        #             print("\n")

        if rank == 0 and step == 0:
            inTag = fr_step.read("tag")
            inNx = fr_step.read("nx")
            inI8 = fr_step.read("gvarI8")
            inI16 = fr_step.read("gvarI16")
            inI32 = fr_step.read("gvarI32")
            inI64 = fr_step.read("gvarI64")
            inU8 = fr_step.read("gvarU8")
            inU16 = fr_step.read("gvarU16")
            inU32 = fr_step.read("gvarU32")
            inU64 = fr_step.read("gvarU64")
            inR32 = fr_step.read("gvarR32")
            inR64 = fr_step.read("gvarR64")

            if inTag != "Testing ADIOS2 high-level API":
                print("InTag: " + str(inTag))
                raise ValueError("tag variable read failed")

            if inNx != nx:
                raise ValueError("tag variable read failed")

            if inI8 != data.i8[0]:
                raise ValueError("gvarI8 read failed")

            if inI16 != data.i16[0]:
                raise ValueError("gvarI16 read failed")

            if inI32 != data.i32[0]:
                raise ValueError("gvarI32 read failed")

            if inI64 != data.i64[0]:
                raise ValueError("gvarI64 read failed")

            if inU8 != data.u8[0]:
                raise ValueError("gvarU8 read failed")

            if inU16 != data.u16[0]:
                raise ValueError("gvarU16 read failed")

            if inU32 != data.u32[0]:
                raise ValueError("gvarU32 read failed")

            if inU64 != data.u64[0]:
                raise ValueError("gvarU64 read failed")

            if inR32 != data.r32[0]:
                raise ValueError("gvarR32 read failed")

            if inR64 != data.r64[0]:
                raise ValueError("gvarR64 read failed")

            # attributes
            inTag = fr_step.read_attribute("attrStr")
            inNx = fr_step.read_attribute("attrNx")
            inI8 = fr_step.read_attribute("attrI8")
            inI16 = fr_step.read_attribute("attrI16")
            inI32 = fr_step.read_attribute("attrI32")
            inI64 = fr_step.read_attribute("attrI64")
            inU8 = fr_step.read_attribute("attrU8")
            inU16 = fr_step.read_attribute("attrU16")
            inU32 = fr_step.read_attribute("attrU32")
            inU64 = fr_step.read_attribute("attrU64")
            inR32 = fr_step.read_attribute("attrR32")
            inR64 = fr_step.read_attribute("attrR64")

            if inTag != "Testing single string attribute":
                raise ValueError("attr string read failed")

            if inNx != nx:
                raise ValueError("attrI8 read failed")

            if inI8 != data.i8[0]:
                raise ValueError("attrI8 read failed")

            if inI16 != data.i16[0]:
                raise ValueError("attrI16 read failed")

            if inI32 != data.i32[0]:
                raise ValueError("attrI32 read failed")

            if inI64 != data.i64[0]:
                raise ValueError("attrI64 read failed")

            if inU8 != data.u8[0]:
                raise ValueError("attrU8 read failed")

            if inU16 != data.u16[0]:
                raise ValueError("attrU16 read failed")

            if inU32 != data.u32[0]:
                raise ValueError("attrU32 read failed")

            if inU64 != data.u64[0]:
                raise ValueError("attrU64 read failed")

            if inR32 != data.r32[0]:
                raise ValueError("attrR32 read failed")

            if inR64 != data.r64[0]:
                raise ValueError("attrR64 read failed")

            # Array attribute
            inTag = fr_step.read_attribute_string("attrStrArray")
            inI8 = fr_step.read_attribute("attrI8Array")
            inI16 = fr_step.read_attribute("attrI16Array")
            inI32 = fr_step.read_attribute("attrI32Array")
            inI64 = fr_step.read_attribute("attrI64Array")
            inU8 = fr_step.read_attribute("attrU8Array")
            inU16 = fr_step.read_attribute("attrU16Array")
            inU32 = fr_step.read_attribute("attrU32Array")
            inU64 = fr_step.read_attribute("attrU64Array")
            inR32 = fr_step.read_attribute("attrR32Array")
            inR64 = fr_step.read_attribute("attrR64Array")

            if inTag != ["string1", "string2", "string3"]:
                raise ValueError("attrStrArray read failed")

            if (inI8 == data.i8).all() is False:
                raise ValueError("attrI8 array read failed")

            if (inI16 == data.i16).all() is False:
                raise ValueError("attrI16 array read failed")

            if (inI32 == data.i32).all() is False:
                raise ValueError("attrI32 array read failed")

            if (inI64 == data.i64).all() is False:
                raise ValueError("attrI64 array read failed")

            if (inU8 == data.u8).all() is False:
                raise ValueError("attrU8 array read failed")

            if (inU16 == data.u16).all() is False:
                raise ValueError("attrU16 array read failed")

            if (inU32 == data.u32).all() is False:
                raise ValueError("attrU32 array read failed")

            if (inU64 == data.u64).all() is False:
                raise ValueError("attrU64 array read failed")

            if (inR32 == data.r32).all() is False:
                raise ValueError("attrR32 array read failed")

            if (inR64 == data.r64).all() is False:
                raise ValueError("attrR64 array read failed")

            inTags = fr_step.read_attribute_string("varattrStrArray", "steps")
            inI8 = fr_step.read_attribute("varattrI8Array", "varI8")
            in16 = fr_step.read_attribute("varattrI16Array", "varI16")
            inI32 = fr_step.read_attribute("varattrI32Array", "varI32")
            inI64 = fr_step.read_attribute("varattrI64Array", "varI64")
            inU8 = fr_step.read_attribute("varattrU8Array", "varU8")
            inU16 = fr_step.read_attribute("varattrU16Array", "varU16")
            inU32 = fr_step.read_attribute("varattrU32Array", "varU32")
            inU64 = fr_step.read_attribute("varattrU64Array", "varU64")
            inR32 = fr_step.read_attribute("varattrR32Array", "varR32")
            inR64 = fr_step.read_attribute("varattrR64Array", "varR64")

            if inTags != ["varattr1", "varattr2", "varattr3"]:
                print(inTags)
                raise ValueError("var attrStrArray read failed")

            if (inI8 == data.i8).all() is False:
                raise ValueError("var attrI8 array read failed")

            if (inI16 == data.i16).all() is False:
                raise ValueError("var attrI16 array read failed")

            if (inI32 == data.i32).all() is False:
                raise ValueError("var attrI32 array read failed")

            if (inI64 == data.i64).all() is False:
                raise ValueError("var attrI64 array read failed")

            if (inU8 == data.u8).all() is False:
                raise ValueError("var attrU8 array read failed")

            if (inU16 == data.u16).all() is False:
                raise ValueError("var attrU16 array read failed")

            if (inU32 == data.u32).all() is False:
                raise ValueError("var attrU32 array read failed")

            if (inU64 == data.u64).all() is False:
                raise ValueError("var attrU64 array read failed")

            if (inR32 == data.r32).all() is False:
                raise ValueError("var attrR32 array read failed")

            if (inR64 == data.r64).all() is False:
                raise ValueError("var attrR64 array read failed")

        stepStr = "Step:" + str(step)

        instepStr = fr_step.read("steps")
        if instepStr != stepStr:
            raise ValueError("steps variable read failed: " + instepStr + " " + stepStr)

        indataI8 = fr_step.read("varI8", start, count)
        indataI16 = fr_step.read("varI16", start, count)
        indataI32 = fr_step.read("varI32", start, count)
        indataI64 = fr_step.read("varI64", start, count)
        indataU8 = fr_step.read("varU8", start, count)
        indataU16 = fr_step.read("varU16", start, count)
        indataU32 = fr_step.read("varU32", start, count)
        indataU64 = fr_step.read("varU64", start, count)
        indataR32 = fr_step.read("varR32", start, count)
        indataR64 = fr_step.read("varR64", start, count)
        fr_step.end_step()

        if (indataI8 == data.i8).all() is False:
            raise ValueError("i8 array read failed")

        if (indataI16 == data.i16).all() is False:
            raise ValueError("i16 array read failed")

        if (indataI32 == data.i32).all() is False:
            raise ValueError("i32 array read failed")

        if (indataI64 == data.i64).all() is False:
            raise ValueError("i64 array read failed")

        if (indataU8 == data.u8).all() is False:
            raise ValueError("u8 array read failed")

        if (indataU16 == data.u16).all() is False:
            raise ValueError("u16 array read failed")

        if (indataU32 == data.u32).all() is False:
            raise ValueError("u32 array read failed")

        if (indataU64 == data.u64).all() is False:
            raise ValueError("u64 array read failed")

        if (indataR32 == data.r32).all() is False:
            raise ValueError("r32 array read failed")

        if (indataR64 == data.r64).all() is False:
            raise ValueError("r64 array read failed")



#!/usr/bin/env python

#
# Distributed under the OSI-approved Apache License, Version 2.0.  See
# accompanying file Copyright.txt for details.
#
# TestBPWriteTypes.py: test Python numpy types in ADIOS2 File Write
#  Created on: Feb 2, 2017
#      Author: William F Godoy godoywf@ornl.gov


from adios2NPTypes import SmallTestData
from mpi4py import MPI
import numpy as np
import adios2.bindings as adios2


def check_object(adios2_object, name):
    if adios2_object is None:
        raise ValueError(str(name) + " not found")


def check_name(name, name_list):
    if name not in name_list:
        raise ValueError(str(name) + " not found in list")


# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()
Nx = 8

# Start ADIOS
adios = adios2.ADIOS(comm)
ioWriter = adios.DeclareIO("writer")

shape = [size * Nx]
start = [rank * Nx]
count = [Nx]

data = SmallTestData()

# ADIOS Variable name, shape, start, offset, constant dims
# All local variables
varI8 = ioWriter.DefineVariable("varI8", data.I8, shape, start, count, adios2.ConstantDims)
varI16 = ioWriter.DefineVariable("varI16", data.I16, shape, start, count, adios2.ConstantDims)
varI32 = ioWriter.DefineVariable("varI32", data.I32, shape, start, count, adios2.ConstantDims)
varI64 = ioWriter.DefineVariable("varI64", data.I64, shape, start, count, adios2.ConstantDims)

varU8 = ioWriter.DefineVariable("varU8", data.U8, shape, start, count, adios2.ConstantDims)
varU16 = ioWriter.DefineVariable("varU16", data.U16, shape, start, count, adios2.ConstantDims)
varU32 = ioWriter.DefineVariable("varU32", data.U32, shape, start, count, adios2.ConstantDims)
varU64 = ioWriter.DefineVariable("varU64", data.U64, shape, start, count, adios2.ConstantDims)

varR32 = ioWriter.DefineVariable("varR32", data.R32, shape, start, count, adios2.ConstantDims)

varR64 = ioWriter.DefineVariable("varR64", data.R64, shape, start, count, adios2.ConstantDims)

attString = ioWriter.DefineAttribute("attrString", ["hello attribute"])
attI8 = ioWriter.DefineAttribute("attrI8", data.I8)

# ADIOS Engine
writer = ioWriter.Open("npTypes.bp", adios2.Mode.Write)

for i in range(0, 3):
    npi8 = np.full((Nx), i, dtype=np.int8)
    npi16 = np.full((Nx), i, dtype=np.int16)
    npi32 = np.full((Nx), i, dtype=np.int32)
    npi64 = np.full((Nx), i, dtype=np.int64)
    npu8 = np.full((Nx), i, dtype=np.uint8)
    npu16 = np.full((Nx), i, dtype=np.uint16)
    npu32 = np.full((Nx), i, dtype=np.uint32)
    npu64 = np.full((Nx), i, dtype=np.uint64)
    npr32 = np.full((Nx), i, dtype=np.float32)
    npr64 = np.full((Nx), i, dtype=np.float64)

    writer.BeginStep()
    writer.Put(varI8, npi8)
    writer.Put(varI16, npi16)
    writer.Put(varI32, npi32)
    writer.Put(varI64, npi64)

    writer.Put(varU8, npu8)
    writer.Put(varU16, npu16)
    writer.Put(varU32, npu32)
    writer.Put(varU64, npu64)

    writer.Put(varR32, npr32)
    writer.Put(varR64, npr64)
    writer.EndStep()

writer.Close()

# Start reader
ioReader = adios.DeclareIO("reader")

reader = ioReader.Open("npTypes.bp", adios2.Mode.ReadRandomAccess)

attrString = ioReader.InquireAttribute("attrString")
attrI8 = ioReader.InquireAttribute("attrI8")

varI8 = ioReader.InquireVariable("varI8")
varI16 = ioReader.InquireVariable("varI16")
varI32 = ioReader.InquireVariable("varI32")
varI64 = ioReader.InquireVariable("varI64")
varU8 = ioReader.InquireVariable("varU8")
varU16 = ioReader.InquireVariable("varU16")
varU32 = ioReader.InquireVariable("varU32")
varU64 = ioReader.InquireVariable("varU64")
varR32 = ioReader.InquireVariable("varR32")
varR64 = ioReader.InquireVariable("varR64")

check_object(attrString, "attrString")
check_object(attrString, "attrI8")

check_object(varI8, "varI8")
check_object(varI16, "varI16")
check_object(varI32, "varI32")
check_object(varI64, "varI64")
check_object(varU8, "varU8")
check_object(varU16, "varU16")
check_object(varU32, "varU32")
check_object(varU64, "varU64")
check_object(varR32, "varR32")
check_object(varR64, "varR64")


attr_names = ["attrString", "attrI8"]
var_names = [
    "varStr",
    "varI8",
    "varI16",
    "varI32",
    "varI64",
    "varU8",
    "varU16",
    "varU32",
    "varU64",
    "varR32",
    "varR64",
]

attributesInfo = ioReader.AvailableAttributes()
for name, info in attributesInfo.items():
    check_name(name, attr_names)
    if rank == 0:
        print("attribute_name: " + name)
        for key, value in info.items():
            print("\t" + key + ": " + value)
        print("\n")

variablesInfo = ioReader.AvailableVariables()
for name, info in variablesInfo.items():
    check_name(name, var_names)
    if rank == 0:
        print("variable_name: " + name)
        for key, value in info.items():
            print("\t" + key + ": " + value)
        print("\n")


if varI8 is not None:
    varI8.SetSelection([[0], [size * Nx]])
    varI8.SetStepSelection([0, 3])
    inI8 = np.zeros((3, size * Nx), dtype=np.int8)
    reader.Get(varI8, inI8)

if varI16 is not None:
    varI16.SetSelection([[0], [size * Nx]])
    varI16.SetStepSelection([0, 3])
    inI16 = np.zeros((3, size * Nx), dtype=np.int16)
    reader.Get(varI16, inI16)

if varI32 is not None:
    varI32.SetSelection([[0], [size * Nx]])
    varI32.SetStepSelection([0, 3])
    inI32 = np.zeros((3, size * Nx), dtype=np.int32)
    reader.Get(varI32, inI32)

if varI64 is not None:
    varI64.SetSelection([[0], [size * Nx]])
    varI64.SetStepSelection([0, 3])
    inI64 = np.zeros((3, size * Nx), dtype=np.int64)
    reader.Get(varI64, inI64)

if varU8 is not None:
    varU8.SetSelection([[0], [size * Nx]])
    varU8.SetStepSelection([0, 3])
    inU8 = np.zeros((3, size * Nx), dtype=np.uint8)
    reader.Get(varU8, inU8)

if varU16 is not None:
    varU16.SetSelection([[0], [size * Nx]])
    varU16.SetStepSelection([0, 3])
    inU16 = np.zeros((3, size * Nx), dtype=np.uint16)
    reader.Get(varU16, inU16)

if varU32 is not None:
    varU32.SetSelection([[0], [size * Nx]])
    varU32.SetStepSelection([0, 3])
    inU32 = np.zeros((3, size * Nx), dtype=np.uint32)
    reader.Get(varU32, inU32)

if varU64 is not None:
    varU64.SetSelection([[0], [size * Nx]])
    varU64.SetStepSelection([0, 3])
    inU64 = np.zeros((3, size * Nx), dtype=np.uint64)
    reader.Get(varU64, inU64)

if varR32 is not None:
    varR32.SetSelection([[0], [size * Nx]])
    varR32.SetStepSelection([0, 3])
    inR32 = np.zeros((3, size * Nx), dtype=np.float32)
    reader.Get(varR32, inR32)

if varR64 is not None:
    varR64.SetSelection([[0], [size * Nx]])
    varR64.SetStepSelection([0, 3])
    inR64 = np.zeros((3, size * Nx), dtype=np.float64)
    reader.Get(varR64, inR64)

reader.PerformGets()

for i in range(0, 3):
    for j in range(0, Nx):
        if inI8[i][j] != i:
            raise ValueError("failed reading I8")

        if inI16[i][j] != i:
            raise ValueError("failed reading I16")

        if inI32[i][j] != i:
            raise ValueError("failed reading I32")

        if inI64[i][j] != i:
            raise ValueError("failed reading I64")

        if inU8[i][j] != i:
            raise ValueError("failed reading U8")

        if inU16[i][j] != i:
            raise ValueError("failed reading U16")

        if inU32[i][j] != i:
            raise ValueError("failed reading U32")

        if inU64[i][j] != i:
            raise ValueError("failed reading U64")

        if inR32[i][j] != i:
            raise ValueError("failed reading R32")

        if inR64[i][j] != i:
            raise ValueError("failed reading R64")


# here tests reader data
reader.Close()



from mpi4py import MPI
import numpy as np
from time import sleep
import adios2.bindings as adios2

# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

# ADIOS MPI Communicator
adios = adios2.ADIOS(comm)

# ADIOS IO
sstIO = adios.DeclareIO("myIO")
sstIO.SetEngine("Sst")

# Python MPI is usually not compatible with MPI used to build ADIOS
# Must use TCP based data transport, or RDMA (ucx, libfabric) if available
sstIO.SetParameter("DataTransport", "WAN")

# note: we need to use np.float32 to be compatible with data from C++ writer
# using "float" works in Python only but leads to type mismatch with C++
myArray = np.array([0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0], dtype=np.float32)
myArray = 10.0 * rank + myArray
nx = len(myArray)
increment = nx * size * 1.0

# ADIOS Variable
ioArray = sstIO.DefineVariable(
    "bpFloats", myArray, [size * nx], [rank * nx], [nx], adios2.ConstantDims
)

sstWriter = sstIO.Open("helloSst", adios2.Mode.Write)
for i in range(4):
    print("Rank=", rank, "loop index =", i, "data =", myArray, flush=True)
    sstWriter.BeginStep()
    sstWriter.Put(ioArray, myArray, adios2.Mode.Sync)
    myArray += increment
    # Warning: the data is not published until EndStep is called
    sstWriter.EndStep()
    sleep(1.0)

sstWriter.Close()



#
# Distributed under the OSI-approved Apache License, Version 2.0.  See
# accompanying file Copyright.txt for details.
#
# dataManReader.py
#
#  Created on: Sept 5, 2019
#      Author: Jason Wang <wangr1@ornl.gov>
#

from mpi4py import MPI
import numpy as np
from adios2 import Adios, Stream
from adios2.bindings import StepStatus

comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

adios = Adios(comm)
io = adios.declare_io("whatever")
io.set_engine("DataMan")
io.set_parameters({"IPAddress": "127.0.0.1", "Port": "12306", "Timeout": "5"})

with Stream(io, "HelloDataMan", "r") as stream:
    for _ in stream.steps():
        stepStatus = stream.step_status()
        print(f"Step status = {stepStatus}")
        var = io.inquire_variable("FloatArray")
        shape = var.shape()
        floatArray = stream.read(var)
        currentStep = stream.current_step()
        loopStep = stream.loop_index()
        print("Loop index =", loopStep, "stream step =", currentStep, "data =")
        print(floatArray)



"""License:
  Distributed under the OSI-approved Apache License, Version 2.0.  See
  accompanying file Copyright.txt for details.
"""

from adios2 import bindings
from adios2.io import IO
from adios2.operator import Operator


class Adios:
    """High level representation of the ADIOS class in the adios2.bindings"""

    def __init__(self, config_file=None, comm=None):
        if comm and not bindings.is_built_with_mpi:
            raise RuntimeError("Cannot use MPI since ADIOS2 was built without MPI support")

        if config_file:
            if comm:
                self.impl = bindings.ADIOS(config_file, comm)
            else:
                self.impl = bindings.ADIOS(config_file)
        else:
            if comm:
                self.impl = bindings.ADIOS(comm)
            else:
                self.impl = bindings.ADIOS()

    @property
    def impl(self):
        """Bindings implementation of the class"""
        return self._impl

    @impl.setter
    def impl(self, implementation):
        self._impl = implementation

    def declare_io(self, name):
        """
        Declare IO instance

        Args:
            name (str): IO instance name
        """
        return IO(self.impl.DeclareIO(name), name, self)

    def at_io(self, name):
        """
        Inquire IO instance

        Args:
            name (str): IO instance name
        Returns:
            IO: IO instance
        """
        io = None
        io_instance = self.impl.AtIO(name)
        if io_instance:
            io = IO(io_instance, name, self)
        return io

    def remove_io(self, name):
        """
        Remove IO instance

        Args:
            name (str): IO instance name
        """
        self.impl.RemoveIO(name)

    def remove_all_ios(self):
        """Remove all IO instance"""
        self.impl.RemoveAllIOs()

    def define_operator(self, name, kind, parameters={}):
        """
        Add an operation (operator).

        Args:
            name (str): name of the operator.
            kind (str): name type of the operation.
            params (dict): parameters as a form of a dict for the operation.
        """
        return Operator(self.impl.DefineOperator(name, kind, parameters), name)

    def inquire_operator(self, name):
        """
        Add an operation (operator).

        Args:
            name (str): name of the operator.
        Return:
            Operator: requested operator
        """
        operator = None
        operator_instance = self.impl.InquireOperator(name)
        if operator_instance:
            operator = Operator(self.impl.InquireOperator(name), name)

        return operator

    def flush_all(self):
        """
        Flush all IO instances
        """
        self.impl.FlushAll()



from paraview.simple import *
from paraview import print_info

# catalyst options
from paraview.catalyst import Options

options = Options()

# print start marker
print_info("begin '%s'", __name__)

# directory under which to save all extracts
# generated using Extractors defined in the pipeline
# (optional, but recommended)
options.ExtractsOutputDirectory = "."
SaveExtractsUsingCatalystOptions(options)

view = CreateRenderView()
# when using Fides, registrationName is always 'fides'
producer = TrivialProducer(registrationName="fides")
display = Show(producer)
view.ResetCamera()
ColorBy(display, ("POINTS", "U"))
display.RescaleTransferFunctionToDataRange(True, False)
display.SetScalarBarVisibility(view, True)
transFunc = GetColorTransferFunction("U")
transFunc.RescaleOnVisibilityChange = 1

display.SetRepresentationType("Surface")

clip = Clip(registrationName="clip1", Input=producer)
clip.ClipType = "Plane"
clip.Scalars = ["POINTS", "U"]
clip.ClipType.Origin = [1.5, 1.5, 1.5]
clip.ClipType.Normal = [0.0, 1.0, 0.0]
clipDisplay = Show(clip, view, "UnstructuredGridRepresentation")

Hide(producer, view)
view.ResetCamera()

camera = GetActiveCamera()
camera.Azimuth(45)
camera.Elevation(45)

# the extractor will save the view on each time step
extractor = CreateExtractor("PNG", view, registrationName="PNG1")
extractor.Writer.FileName = "output-{timestep}.png"
extractor.Writer.ImageResolution = [800, 800]


def catalyst_execute(info):
    print_info("in '%s::catalyst_execute'", __name__)
    global producer
    producer.UpdatePipeline()
    print("updating pipeline and saving image")

    # print("-----------------------------------")
    # print("executing (cycle={}, time={})".format(info.cycle, info.time))
    # print("bounds:", producer.GetDataInformation().GetBounds())
    # print("U-range:", producer.PointData['U'].GetRange(0))
    # print("V-range:", producer.PointData['V'].GetRange(0))


# print end marker
print_info("end '%s'", __name__)



#
from mpi4py import MPI
import numpy as np
import adios2.bindings as adios2
import sys

# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

# #######################################
# #  usage: <exe> [bp4 | bp5=default]  ##
# #######################################
numSteps = 5
queryFile = "query.xml"
targetVarName = "var0"

# User data
myArray = np.array([0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0])
Nx = myArray.size

# ADIOS MPI Communicator
adios = adios2.ADIOS(comm)

supportedEngines = ["bp5", "bp4"]
engineType = "bp5"
if len(sys.argv) > 1:
    engineType = sys.argv[1].lower()

if engineType in supportedEngines:
    if rank == 0:
        print("Using engine type:", engineType)
else:
    sys.exit("specified engine does not exist")


dataFileName = "test_" + engineType + ".bp"


def writeDataFile():
    bpIO = adios.DeclareIO("Writer")
    bpIO.SetEngine(engineType)

    ioArray = bpIO.DefineVariable(targetVarName, myArray, [], [], [Nx], False)

    bpIO.SetParameter("statsblocksize", "3")

    bpFileWriter = bpIO.Open(dataFileName, adios2.Mode.Write)

    for i in range(numSteps):
        bpFileWriter.BeginStep()
        bpFileWriter.Put(ioArray, i * 10.0 + myArray / (rank + 1), adios2.Mode.Sync)
        bpFileWriter.EndStep()

    bpFileWriter.Close()


def createQueryFile():
    print(".. Writing query file to: ", queryFile)

    file1 = open(queryFile, "w")
    queryContent = [
        '<?xml version="1.0"?>\n',
        "<adios-query>\n",
        '  <io name="query">\n' '  <var name="' + targetVarName + '">\n',
        '    <op value="AND">\n',
        '      <range  compare="LT" value="15.0"/>\n',
        '      <range  compare="GT" value="4.0"/>\n',
        "    </op>\n",
        "  </var>\n",
        "  </io>\n",
        "</adios-query>\n",
    ]
    file1.writelines(queryContent)
    file1.close()


def doAnalysis(reader, touched_blocks, varList):
    print(" Step: ", reader.CurrentStep(), "  num touched blocks: ", len(touched_blocks))

    values = []
    data = {}

    for var in varList:
        data[var] = []

    if len(touched_blocks) > 0:
        for n in touched_blocks:
            for var in varList:
                values = np.zeros(10, dtype=np.double)
                var.SetBlockSelection(n)
                reader.Get(var, values, adios2.Mode.Sync)
                data[var].extend(values)


def queryDataFile():
    # # use no mpi
    adios_nompi = adios2.ADIOS()
    queryIO = adios_nompi.DeclareIO("query")

    reader = queryIO.Open(dataFileName, adios2.Mode.Read)
    print("dataFile=", dataFileName, "queryFile=", queryFile)
    touched_blocks = []

    print("Num steps: ", reader.Steps())

    while reader.BeginStep() == adios2.StepStatus.OK:
        # bp5 loads metadata after beginstep(),
        # therefore query has to be called per step
        w = adios2.Query(queryFile, reader)
        # assume only rank 0 wants to process result
        var = [queryIO.InquireVariable(targetVarName)]

        if rank == 0:
            touched_blocks = w.GetBlockIDs()
            doAnalysis(reader, touched_blocks, var)

        reader.EndStep()
    reader.Close()


def cleanUp():
    import os
    import shutil

    os.remove(queryFile)
    shutil.rmtree(dataFileName)
    print("  Cleanup generated files: ", queryFile, dataFileName)


#
# actual setup:
#


writeDataFile()


if 0 == rank:
    createQueryFile()
    queryDataFile()
    cleanUp()



#
# Distributed under the OSI-approved Apache License, Version 2.0.  See
# accompanying file Copyright.txt for details.
#
# bpReaderHeatMap2D.py
#
#
#  Created on: Dec 5th, 2017
#      Author: William F Godoy godoywf@ornl.gov
#              Norbert Podhorszki pnorbert@ornl.gov
#

from mpi4py import MPI
import numpy
from adios2 import Stream, FileReader

# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

# User data
Nx = 10
Ny = 10

count = [Nx, Ny]
start = [rank * Nx, 0]
shape = [size * Nx, Ny]

temperatures = numpy.zeros(Nx * Ny, dtype=int)

for i in range(0, Nx):
    iGlobal = start[0] + i

    for j in range(0, Ny):
        value = iGlobal * shape[1] + j
        temperatures[i * Nx + j] = value

with Stream("HeatMap2D_py.bp", "w", comm) as obpStream:
    obpStream.write("temperature2D", temperatures, shape, start, count)
    if not rank:
        obpStream.write("N", [size, Nx, Ny])  # will be an array in output
        obpStream.write("Nx", numpy.array(Nx))  # will be a scalar in output
        obpStream.write("Ny", Ny)  # will be a scalar in output
        obpStream.write_attribute("nproc", size)  # will be a single value attribute in output
        obpStream.write_attribute("dimensions", [size * Nx, Ny], "temperature2D")

if not rank:
    with FileReader("HeatMap2D_py.bp", MPI.COMM_SELF) as ibpFile:
        # scalar variables are read as a numpy array with 0 dimension
        in_nx = ibpFile.read("Nx")
        in_ny = ibpFile.read("Ny")
        print(f"Incoming nx, ny = {in_nx}, {in_ny}")

        # single value attribute is read as a numpy array with 0 dimension
        in_nproc = ibpFile.read_attribute("nproc")
        print(f"Incoming nproc = {in_nproc}")
        # array attribute is read as a numpy array or string list
        in_dims = ibpFile.read_attribute("temperature2D/dimensions")
        print(f"Incoming diumensions = {in_dims}")

        # On option is to inquire a variable to know its type, shape
        # directly, not as strings, and then we can use the variable
        # object to set selection and/or set steps to read
        var_inTemperature = ibpFile.inquire_variable("temperature2D")
        if var_inTemperature is not None:
            var_inTemperature.set_selection([[2, 2], [4, 4]])
            inTemperatures = ibpFile.read(var_inTemperature)
            print(
                f"Incoming temperature map with selection "
                f"start = {var_inTemperature.start()}, count = {var_inTemperature.count()}"
            )
            for i in range(0, inTemperatures.shape[1]):
                print(str(inTemperatures[i]))



#!/usr/bin/env python

#
# Distributed under the OSI-approved Apache License, Version 2.0.  See
# accompanying file Copyright.txt for details.
#
# TestBPWriteTypes.py: test Python numpy types in ADIOS2 File Write
#  Created on: Feb 2, 2017
#      Author: William F Godoy godoywf@ornl.gov


from adios2NPTypes import SmallTestData
from mpi4py import MPI
import adios2.bindings as adios2


def check_object(adios2_object, name):
    if adios2_object is False:
        raise ValueError(str(name) + " not found")


def check_name(name, name_list):
    if name not in name_list:
        raise ValueError(str(name) + " not found in list")


def check_array(np1, np2, hint):
    if (np1 == np2).all() is False:
        print("InData: " + str(np1))
        print("Data: " + str(np2))
        raise ValueError("Array read failed " + str(hint))


# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()
Nx = 8

# list of tested attributes and variables
attr_names = [
    "attrString",
    "attrStringArray",
    "attrI8",
    "attrI16",
    "attrI32",
    "attrI64",
    "attrU8",
    "attrU16",
    "attrU32",
    "attrU64",
    "varR32/attrR32",
    "varR64::attrR64",
]
var_names = [
    "varStr",
    "varI8",
    "varI16",
    "varI32",
    "varI64",
    "varU8",
    "varU16",
    "varU32",
    "varU64",
    "varR32",
    "varR64",
]

# Start ADIOS
adios = adios2.ADIOS(comm)
ioWriter = adios.DeclareIO("writer")

shape = [size * Nx]
start = [rank * Nx]
count = [Nx]

data = SmallTestData()

# ADIOS Variable name, shape, start, offset, constant dims
# All local variables
varStr = ioWriter.DefineVariable("varStr")

varI8 = ioWriter.DefineVariable("varI8", data.I8, shape, start, count, adios2.ConstantDims)
varI16 = ioWriter.DefineVariable("varI16", data.I16, shape, start, count, adios2.ConstantDims)
varI32 = ioWriter.DefineVariable("varI32", data.I32, shape, start, count, adios2.ConstantDims)
varI64 = ioWriter.DefineVariable("varI64", data.I64, shape, start, count, adios2.ConstantDims)

varU8 = ioWriter.DefineVariable("varU8", data.U8, shape, start, count, adios2.ConstantDims)
varU16 = ioWriter.DefineVariable("varU16", data.U16, shape, start, count, adios2.ConstantDims)
varU32 = ioWriter.DefineVariable("varU32", data.U32, shape, start, count, adios2.ConstantDims)
varU64 = ioWriter.DefineVariable("varU64", data.U64, shape, start, count, adios2.ConstantDims)

varR32 = ioWriter.DefineVariable("varR32", data.R32, shape, start, count, adios2.ConstantDims)

varR64 = ioWriter.DefineVariable("varR64", data.R64, shape, start, count, adios2.ConstantDims)

attString = ioWriter.DefineAttribute("attrString", "one")
attStringArray = ioWriter.DefineAttribute("attrStringArray", ["one", "two", "three"])
attI8 = ioWriter.DefineAttribute("attrI8", data.I8)
attI16 = ioWriter.DefineAttribute("attrI16", data.I16)
attI32 = ioWriter.DefineAttribute("attrI32", data.I32)
attI64 = ioWriter.DefineAttribute("attrI64", data.I64)
attU8 = ioWriter.DefineAttribute("attrU8", data.U8)
attU16 = ioWriter.DefineAttribute("attrU16", data.U16)
attU32 = ioWriter.DefineAttribute("attrU32", data.U32)
attU64 = ioWriter.DefineAttribute("attrU64", data.U64)
# add an attribute to a variable
attR32 = ioWriter.DefineAttribute("attrR32", data.R32, "varR32")
attR64 = ioWriter.DefineAttribute("attrR64", data.R64, "varR64", "::")

ioWriter.SetEngine("BPFile")
ioParams = {"Threads": "1", "InitialBufferSize": "17Kb"}
ioWriter.SetParameters(ioParams)

engineType = ioWriter.EngineType()
if engineType != "BPFile":
    raise ValueError(str(engineType) + " incorrect engine type, should be BPFile")

ioWriter.SetParameter("profileunits", "microseconds")
ioWriter.AddTransport("file")

ioParams = ioWriter.Parameters()
print("Final IO parameters")
for key, value in ioParams.items():
    print("\t" + key + ": " + value)

nsteps = 3

# ADIOS Engine
writer = ioWriter.Open("npTypes.bp", adios2.Mode.Write)

writer.LockWriterDefinitions()

for i in range(0, nsteps):
    data.update(rank, i, size)

    writer.BeginStep()
    writer.Put(varStr, data.Str)
    writer.Put(varI8, data.I8)
    writer.Put(varI16, data.I16)
    writer.Put(varI32, data.I32)
    writer.Put(varI64, data.I64)

    writer.Put(varU8, data.U8)
    writer.Put(varU16, data.U16)
    writer.Put(varU32, data.U32)
    writer.Put(varU64, data.U64)

    writer.Put(varR32, data.R32)
    writer.Put(varR64, data.R64)
    writer.EndStep()

writer.Close()

# Start reader
ioReader = adios.DeclareIO("reader")

reader = ioReader.Open("npTypes.bp", adios2.Mode.ReadRandomAccess)

attrString = ioReader.InquireAttribute("attrString")
attrStringArray = ioReader.InquireAttribute("attrStringArray")
attrI8 = ioReader.InquireAttribute("attrI8")
attrI16 = ioReader.InquireAttribute("attrI16")
attrI32 = ioReader.InquireAttribute("attrI32")
attrI64 = ioReader.InquireAttribute("attrI64")
attrU8 = ioReader.InquireAttribute("attrU8")
attrU16 = ioReader.InquireAttribute("attrU16")
attrU32 = ioReader.InquireAttribute("attrU32")
attrU64 = ioReader.InquireAttribute("attrU64")
attrR32 = ioReader.InquireAttribute("attrR32", "varR32")
attrR64 = ioReader.InquireAttribute("attrR64", "varR64", "::")

check_object(attrString, "attrString")
check_object(attrStringArray, "attrStringArray")
check_object(attrI8, "attrI8")
check_object(attrI16, "attrI16")
check_object(attrI32, "attrI32")
check_object(attrI64, "attrI64")
check_object(attrU8, "attrU8")
check_object(attrU16, "attrU16")
check_object(attrU32, "attrU32")
check_object(attrU64, "attrU64")
check_object(attrR32, "varR32/attrR32")
check_object(attrR64, "varR64::attrR64")

# alternative inquire format
attrR32 = ioReader.InquireAttribute("varR32/attrR32")
attrR64 = ioReader.InquireAttribute("varR64::attrR64")
check_object(attrR32, "varR32/attrR32")
check_object(attrR64, "varR64::attrR64")

attrStringData = attrString.DataString()
print(f"attrString = {attrStringData}", flush=True)
if attrStringData[0] != "one":
    raise ValueError("attrString failed")

attrStringData = attrStringArray.DataString()
print(f"attrStringArray = {attrStringData}", flush=True)
if attrStringData[0] != "one":
    raise ValueError("attrStringData[0] failed")
if attrStringData[1] != "two":
    raise ValueError("attrStringData[1] failed")
if attrStringData[2] != "three":
    raise ValueError("attrStringData[2] failed")

attrI8Data = attrI8.Data()
attrI16Data = attrI16.Data()
attrI32Data = attrI32.Data()
attrI64Data = attrI64.Data()
attrU8Data = attrU8.Data()
attrU16Data = attrU16.Data()
attrU32Data = attrU32.Data()
attrU64Data = attrU64.Data()
attrR32Data = attrR32.Data()
attrR64Data = attrR64.Data()

check_array(attrI8Data, data.I8, "I8")
check_array(attrI16Data, data.I16, "I16")
check_array(attrI32Data, data.I32, "I32")
check_array(attrI64Data, data.I64, "I64")
check_array(attrU8Data, data.U8, "U8")
check_array(attrU16Data, data.U16, "U16")
check_array(attrU32Data, data.U32, "U32")
check_array(attrU64Data, data.U64, "U64")
check_array(attrR32Data, data.R32, "R32")
check_array(attrR64Data, data.R64, "R64")

print("=========== Attributes ===========")
attributesInfo = ioReader.AvailableAttributes()
for name, info in attributesInfo.items():
    check_name(name, attr_names)
    if rank == 0:
        print("attribute_name: " + name)
        for key, value in info.items():
            print("\t" + key + ": " + value)
        print("\n")

print("=========== Available attributes of varR32 ===========")
attributesInfoR32 = ioReader.AvailableAttributes("varR32")
for name, info in attributesInfoR32.items():
    check_name(name, ["attrR32"])
    if rank == 0:
        print("attribute_name: " + name)
        for key, value in info.items():
            print("\t" + key + ": " + value)
        print("\n")

print("=========== Available attributes of varR64 ===========")
attributesInfoR32 = ioReader.AvailableAttributes("varR64", "::")
for name, info in attributesInfoR32.items():
    check_name(name, ["attrR64"])
    if rank == 0:
        print("attribute_name: " + name)
        for key, value in info.items():
            print("\t" + key + ": " + value)
        print("\n")

print("=========== Variables ===========")
varStr = ioReader.InquireVariable("varStr")
varI8 = ioReader.InquireVariable("varI8")
varI16 = ioReader.InquireVariable("varI16")
varI32 = ioReader.InquireVariable("varI32")
varI64 = ioReader.InquireVariable("varI64")
varU8 = ioReader.InquireVariable("varU8")
varU16 = ioReader.InquireVariable("varU16")
varU32 = ioReader.InquireVariable("varU32")
varU64 = ioReader.InquireVariable("varU64")
varR32 = ioReader.InquireVariable("varR32")
varR64 = ioReader.InquireVariable("varR64")

check_object(varStr, "varStr")
check_object(varI8, "varI8")
check_object(varI16, "varI16")
check_object(varI32, "varI32")
check_object(varI64, "varI64")
check_object(varU8, "varU8")
check_object(varU16, "varU16")
check_object(varU32, "varU32")
check_object(varU64, "varU64")
check_object(varR32, "varR32")
check_object(varR64, "varR64")

variablesInfo = ioReader.AvailableVariables()
for name, info in variablesInfo.items():
    check_name(name, var_names)
    if rank == 0:
        print("variable_name: " + name)
        for key, value in info.items():
            print("\t" + key + ": " + value)
        print("\n")


result = adios.RemoveIO("writer")
if result is False:
    raise ValueError("Could not remove IO writer")

assert reader.Steps() == nsteps


reader.Close()

ioReader.RemoveAllVariables()
varStr = ioReader.InquireVariable("varStr")
if varStr is True:
    raise ValueError("Could remove reader variables")

adios.RemoveAllIOs()
try:
    ioWriter = adios.DeclareIO("reader")
except ValueError:
    raise ValueError("Could not re-Declare IO reader")
